"""CLI console package: event rendering, lifecycle panels, and block type renderers."""

from yera.console.block_renderers import (
    render_action,
    render_buttons,
    render_date,
    render_input_request,
    render_markdown,
    render_slider,
    render_spinner,
    render_table,
)
from yera.console.console_renderer import ConsoleRenderer
from yera.console.panels import agent_panel, exit_panel, startup_panel
from yera.console.render_result import AwaitUserResult, BlockResult, InformationalResult

__all__ = [
    "AwaitUserResult",
    "BlockResult",
    "ConsoleRenderer",
    "InformationalResult",
    "agent_panel",
    "exit_panel",
    "render_action",
    "render_buttons",
    "render_date",
    "render_input_request",
    "render_markdown",
    "render_slider",
    "render_spinner",
    "render_table",
    "startup_panel",
]
